package express.wrapper

external fun require(module: String): dynamic

object Express {

    fun create(): ExpressApp {
        val express = require("express")
        return express() as ExpressApp
    }
}

@Suppress("unused")
external class ExpressApp {

    val locals: dynamic
    val mountpath: dynamic

    fun use(extension: dynamic)

    fun get(endpoint: String, func: (Request, Response) -> Unit)
    fun post(endpoint: String, func: (Request, Response) -> Unit)
    fun put(endpoint: String, func: (Request, Response) -> Unit)

    fun listen(port: Int)
    fun listen(port: Int, func: () -> Unit)
}

@Suppress("unused")
external class Request {

    // This property holds a reference to the instance of the Express application
    // that is using the middleware
    val app: ExpressApp


    // Contains key-value pairs of data submitted in the request body.
    // By default, it is undefined, and is populated when you use body-parsing
    // middleware such as body-parser and multer.
    val body: dynamic

    // When using cookie-parser middleware, this property is an object that contains
    // cookies sent by the request.
    // If the request contains no cookies, it defaults to {}.
    val cookies: dynamic

    // Signed cookies reside in a different object to show developer intent;
    // otherwise, a malicious attack could be placed on req.cookie values
    val signedCookies: dynamic


    // Indicates whether the request is "fresh", it is the opposite of "stale".
    val fresh: Boolean
    // Indicates whether the request is "stale", and is the opposite of "fresh".
    val stale: Boolean


    // Contains the hostname derived from the Host HTTP header.
    val hostname: String

    // Contains the remote IP address of the request.
    val ip: String
    // When the trust proxy setting does not evaluate to false, this property contains an array of IP addresses
    // specified in the X-Forwarded-For request header.
    // Otherwise, it contains an empty array.
    // This header can be set by the client or by the proxy
    val ips: Array<String>


    // GET, POST, PUT
    val method: String

    // This property is much like req.url;
    // however, it retains the original request URL,
    // allowing you to rewrite req.url freely for internal routing purposes.
    //
    // GET      'http://www.example.com/admin/new'
    // Example  '/admin/new'
    val originalUrl: String
    // The URL path on which a router instance was mounted
    //
    // GET      'http://www.example.com/admin/new'
    // Example  '/admin'
    val baseUrl: String
    // Contains the path part of the request URL
    //
    // GET      'http://www.example.com/admin/new'
    // Example  '/new'
    val path: String

    // This property is an object containing properties mapped to the named route “parameters”.
    // For example, if you have the route /user/:name,
    // then the “name” property is available as req.params.name
    val params: String

    // Contains the request protocol string: either http or (for TLS requests) https
    val protocol: String

    // This property is an object containing a property for each query string parameter
    // in the route.
    // If there is no query string, it is the empty object, {}.
    //
    // GET      '/search?q=tobi+ferret'
    // Example  'tobi ferret'
    val query: String

    // Contains the currently-matched route, a string
    val route: String

    // A Boolean property that is true if a TLS connection is established
    // 'https' == req.protocol
    val secure: Boolean

    // An array of subdomains in the domain name of the request
    // Host:    "tobi.ferrets.example.com"
    // => ["ferrets", "tobi"]
    val subdomain: Array<String>

    // A Boolean property that is true if the request’s X-Requested-With header field
    // is “XMLHttpRequest”, indicating that the request was issued by a
    // client library such as jQuery
    val xhr: Boolean


    fun accept(type: String): dynamic
    fun accept(types: Array<String>): dynamic

    fun get(field: String): dynamic

    fun `is`(type: String): dynamic

    fun param(name: String): dynamic
}

@Suppress("unused")
external class Response {

    // This property holds a reference to the instance of the Express application
    // that is using the middleware.
    val app: ExpressApp

    // Boolean property that indicates if the app sent HTTP headers for the response.
    val headersSent: Boolean

    // An object that contains response local variables scoped to the request,
    // and therefore available only to the view(s) rendered during that
    // request / response cycle (if any).
    //
    // Otherwise, this property is identical to app.locals.
    val locals: dynamic

    // Appends the specified value to the http response header field.
    // If the header is not already set, it creates the header with the specified value.
    // The value parameter can be a string or an array.
    fun append(name: String, value: Any)

    // Sets the http response content-disposition header filed to "attachment".
    // If a filename is given then it sets the Content-type based on the extension name
    // via res.type()
    fun attachment()
    fun attachment(path: String)

    // Ends the response process. This method actually comes from Node core, specifically
    // the response.end() method of http.ServerResponse.
    fun end()

    // Returns the http response header specified by field. the match is case-insensitive
    fun get(field: String): dynamic

    // Sets the HTTP status for the response. It is a chainable alias of Node's
    // response.statusCode
    fun status(code: Int): Response

    // Sets the Content-Type HTTP header to the MIME type as determined by mime.lookup()
    // for the specified type. If type contains the "/" character, then it sets the Content-Type to type
    fun type(type: String)

    // Sends the Http response.
    fun send(response: Any)
}